#!/usr/bin/perl

use Getopt::Std;

my %RunningCrons = GetRunningCrons();

if ( !getopts('k:') ) {
    print "USAGE: nachtls.pl [-k <release>]\n";
    print "\n";
    print "       shows running nachtRelease jobs or kills them\n";
    print "\n";
	foreach my $Rel (keys %RunningCrons) {
		print "       nachtRelease $Rel still running (Pid:$RunningCrons{$Rel})\n";
	}
    exit 1;
}


if ( $opt_k ) {

	if ( $RunningCrons{$opt_k} ) {
		print "Killing releas $opt_k (Pid:$RunningCrons{$opt_k}))\n";
		PauseCron ($RunningCrons{$opt_k});
		KillCron ($RunningCrons{$opt_k});
	} else {
		print "unknown or not running release: $opt_k\n"
	}

} else {

	if (%RunningCrons) {
		foreach my $Rel (keys %RunningCrons) {
			print "nachtRelease $Rel still running (Pid:$RunningCrons{$Rel})\n";
		}
	} else {
		print "no nachtRelease running\n";
	}

}
########################################
sub GetRunningCrons ()
{
	my %CronHash;

	my ($CronPid) = split ' ', `ps -e | grep cron`;

	my @NachtProcs = grep { s/^\s*remuser\s+(\d+)\s+$CronPid\s+.*nachtRelease\.bash\s+(\d*)\s+.*1>.*$/$1 $2/ } `ps -ef`;

	foreach my $NachtProc (@NachtProcs) {

		my ($Pid, $Release) = split ( ' ', $NachtProc );
		$CronHash{$Release} = $Pid;

	}

	return %CronHash;
}

########################################
sub PauseCron ()
{
	my (@ProcList) = @_;

    while (@ProcList) {
		my $ActPid = shift @ProcList;
		kill 'STOP', $ActPid;
		push @ProcList , GetChildPids($ActPid);
	}

}

########################################
sub KillCron ()
{
	my (@ProcList) = @_;

    while (@ProcList) {
		my $ActPid = shift @ProcList;
		push @ProcList , GetChildPids($ActPid);
		kill 'KILL', $ActPid;
	}

}

########################################
sub GetChildPids ()
{
	my ($ParentPid) = @_;

	my @ChildPids;
    my @psout = `ps -ef | grep remuser | grep $ParentPid`;
    foreach my $ps ( @psout ) {
   		push (@ChildPids, $1) if ( $ps =~ /.*\s+(\d+)\s+$ParentPid\s+.+$/ );
	}

    return @ChildPids;

}

