#!/usr/bin/perl
#
#    ========== licence begin  GPL
#    Copyright (C) 2001 SAP AG
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#    ========== licence end
#

#
#    @(#)toolvars-tml.pl
#    included by several scripts, defines system specific values
#


use Env;
use Config;
use CCVers;
use POSIX;

# CTS 1110865
@THREAD_CCFLAGS=();
@THREAD_LDFLAGS=();
@LDFLAGS=();
@LDXXFLAGS=();

if ($RELVER eq "")
{
	print "\n!!! WARNING:  RELVER not set!!!\n";
	exit 1;
}

if (! $ENV{INSTROOT})
{
	print "\n!!! WARNING:  INSTROOT not set!!!\n";
	exit 1;
}

# force set of compiler version
CCVers::setup_compiler_version();


$ADABAS_COPYRIGHT="Copyright  SAP AG 1997";
if ($RELVER eq "R61") {
	$ADABAS_VERSION="6.1";
}
elsif ($RELVER eq "R62") {
	$ADABAS_VERSION="6.2";
}
elsif ($RELVER eq "R70") {
	$ADABAS_VERSION="7.0";
}
elsif ($RELVER eq "R71") {
	$ADABAS_VERSION="7.1";
}
elsif ($RELVER eq "R72") {
	$ADABAS_VERSION="7.2";
}
elsif ($RELVER eq "R73") {
	$ADABAS_VERSION="7.3";
}
elsif ($RELVER eq "R74") {
	$ADABAS_VERSION="7.4";
}
elsif ($RELVER eq "R80") {
	$ADABAS_VERSION="8.0";
}


$SQLVERSION="REL30";$ENV{SQLVERSION} = $SQLVERSION; # export $SQLVERSION

$OSSPEC=HPUX;
$ENV{OSSPEC} = $OSSPEC;		# export $OSSPEC
#
 if ($ENV{BIT64} eq "1") {
   $MACH = "PA20W";
 } else {
   if ( defined $ENV{MACH} ) {
     $MACH = $ENV{MACH};
     if ( $MACH eq "PA20W" ) {
       $ENV{BIT64} = "1";
	 } else {
       delete $ENV{BIT64};
	 }
   } else {
	 $MACH = "PA11";
     delete $ENV{BIT64};
   }
 }
#
# End of OSSPEC = HPUX specific MACH/BIT64 settings...
#
$ENV{MACH} = $MACH;		# export $MACH
$OS = "UNIX";
$ENV{OS} = $OS;			# export $OS

# CTS 1106697
unless ( $ENV{OS_VER} && $ENV{OS_REL} )
{
    local
    ($os_sysname, $os_nodename, $os_release, $os_version, $os_machine) =
    POSIX::uname ();
    $ENV{OS_VER} = $os_version;
    $ENV{OS_REL} = $os_release;
}
$OS_VER = $ENV{OS_VER};
$OS_REL = $ENV{OS_REL};


if ($ENV{BIT64} eq "1") {
	$TOOL_LIB = "$TOOL/lib/lib64";
} else {
	$TOOL_LIB = "$TOOL/lib";
}
$ENV{TOOL_LIB} = $TOOL_LIB;

$COMP="PTOC";
if ($RELVER eq "R61") {
	#@CONCOMOPTS = ( "-REL=R61" );
	@CONCOMOPTS = ( "-$SQLVERSION=1", "-REL=R61" );
	#@CONCOMOPTS = ( "-SAG=1", "-$SQLVERSION=1", "-REL=R61" );
}
else
{
	#@CONCOMOPTS = ( );
	@CONCOMOPTS = ( "-$SQLVERSION=1" );
	#@CONCOMOPTS = ( "-SAG=1", "-$SQLVERSION=1" );
}
push @CONCOMOPTS, "-RELVER=$RELVER";
push @CONCOMOPTS, "-DIAGNOSE=1" if defined( $ENV{DIAGNOSE} );
#
# Used for all systems using 64 bit compilation!
#
if ( $ENV{BIT64} eq "1" )
{
	push @CONCOMOPTS, "-BIT64=1";
	if ( $ENV{XCOMP} eq "1" )
	{
		push @CONCOMOPTS, "-XCOMP=1";
	}
}
# used for 32 KB pages  (PTS 1111111)
if ( $ENV{page32KB} )
{
	push @CONCOMOPTS, "-page32KB=1";
}

# jrg 2.9.1999

if ( not $CC )
{
    $CC = "cc";
}
#else take CC from the environment
if ( not $CXX )
{
	$CXX = "aCC";
}




# CTS 1107045
# $LDDLL_C         - command of link c objects to shared lib
# $LDDLL_CXX       - command of link c++ shared lib
# $LDDLL_CXX_C     - command of link c++ objects to c shared lib
# to initialize $LDDLL
if ($RELVER ge "R72")
{	$LDDLL_C = "ld"; }
else
{   $LDDLL_C = $CC; }
$LDDLL_CXX = $CXX;
$LDDLL_CXX_C = "ld";
#else take CXX from the environment PTS 1001861

if (($RELVER ge "R72") && ($RELVER ne "RTOOL"))  # &gar PTS 1002110
{
	$LD = $CXX;
	$LDDLL_TYPE = "LDDLL_CXX";
}
else
{
	$LD	= $CC;
	$LDDLL_TYPE = "LDDLL_C";
}

# jrg 2.9.1999
# For relocatable objects (collection of object files) a different linker is used
# These definitions replace former system dependend setting in linkrel.pl!
#
# WIN32 uses $AR and $ARFLAGS...
$LDREL = "ld";

if ($MACH eq "HP9")
{
	$AS		= "as";
}
elsif (($MACH eq "PA11") || ($MACH eq "PA20W"))
{
	$AS		= $CC;
}
$AR		= "ar";
$NULL = "/dev/null";
# set C and CXX as environment variable
$ENV{CC}=$CC;
$ENV{CXX}=$CXX;

#
# Additional compiler flags may be set temporary in the environment
# before calling vmake.
#

@ASFLAGS = ();

# 98/06/19 &gar: for differences between C and C++-Compiling in UNIX !
# @CCFLAGS   -> FLAGS for both ( C, C++ )
# @CFLAGS    -> FLAGS for C only
# @CXXFLAGS  -> FLAGS for C++ only
  @CXXFLAGS = split /\s+/, $CXXFLAGS;
  @CFLAGS = split /\s+/, $CFLAGS;
@CCFLAGS = split /\s+/, $CCFLAGS;
push @CCFLAGS, "-D$SQLVERSION",  "-D$OSSPEC", "-D$MACH", "-DSAG";
push @CCFLAGS, "-DDIAGNOSE" if defined( $ENV{DIAGNOSE} );
push @CCFLAGS, "-DSAPDBMem_USE_SYSTEM_ALLOC" if defined( $ENV{SAPDBMem_USE_SYSTEM_ALLOC} );
@CCMODEANSI = ();
@CCMODENORM = ();
@CCMODELINT = ();

if ($ENV{APOMAKE})
{
	push @CCFLAGS,  "-DAPO_COM_SHORT=$ENV{APO_COM_SHORT}";

	push @CCFLAGS,  "-DAPO_PATCH_LEVEL=$ENV{APO_PATCH_LEVEL}" if ($ENV{APO_PATCH_LEVEL});
	#  "-DAPO_COM_RELEASE=$APO_COM_RELEASE",
	#  "-DAPO_COM_BUILD=$APO_COM_BUILD",


	push @CCFLAGS,  "-DBRANCH_INFO=$ENV{BRANCH_INFO}",
	                "-DLATEST_CHANGE_LIST=$ENV{LATEST_CHANGE_LIST}";
#	                "-DBRANCH_INFO_STR=\"$BRANCH_INFO\"",
#					"-DLATEST_CHANGE_LIST_STR=\"$LATEST_CHANGE_LIST\"";

	# BRANCH_INFO, BRANCH_INFO
	push @CCFLAGS, "-DSAPonUNIX", "-DSAPwithTHREADS";
	push @CCFLAGS, "-DSAP_JS_OS_VERSION=$OS_VER", "-DSAP_JS_OS_RELEASE=$OS_REL";
	push @CCFLAGS, "-D__HP_NO_MATH_OVERLOADS", "-DSAPonHPPA", "+Z";
	push @CCFLAGS, "+DA2.0W", "+DA2.0", "-D_REENTRANT" ,"-w";
	if ($ENV{VMAKE_CURRENT_VARIANT} =~ /Release/ )
	{ push @CCFLAGS, "-DNDEBUG" }
	elsif ($ENV{VMAKE_CURRENT_VARIANT} =~ /Debug/ )
	{ push @CCFLAGS, "-DDEBUG" }
}


# 1.8.2001 E.Z. PTS 1111111
if ( $ENV{page32KB} )
{
   push @CCFLAGS, "-Dpage32KB";
}


if ( $RELVER ge "R80" ) {
#
# gg 22.12.2001 put stlport directory in include search path
#
    push @CCFLAGS, "-I$WRK/incl/SAPDB/STLport/stlport";
#
# Some day this can be removed again if we can use src/SAPDB as new src...
#
    push @CCFLAGS, "-I$WRK/incl/SAPDB";
}
@EXCEPT_FLAG = ();
@RTTI_FLAG = ();    # on by default

# check for LC_INC for APO
die "LC_INC is not defined in environment!\n" if ( $ENV{APOMAKE} && ! $ENV{LC_INC} );
push @CCFLAGS, "-I$ENV{LC_INC}" if ( $ENV{APOMAKE} && $ENV{LC_INC} );


  if ( ($RELVER ge "R72") && ($RELVER ne "RTOOL")){

push @THREAD_CCFLAGS, "-D_REENTRANT";		# CTS 1110865
       push @CCFLAGS, "-I$INSTROOT/incl", "-I$WRK/incl";

  } else {
#
# older versions ( < V7.2 ) do not support multithreading in any way...
#
    push @CCFLAGS, "-I$INSTROOT/incl", "-I$WRK/incl";

  }

#
# Some day this can be removed again if we can use src/SAPDB as new src...
#
  if (( $RELVER ge "R73" ) && ( ! $ENV{APOMAKE} )) {
  		push @CCFLAGS, "-I$WRK/incl/SAPDB";
    }


if ($MACH eq "HP9") {
	@LINTFLAGS = @CCFLAGS;
#PTS 1105844
#PTS 1106628
    if (( $RELVER ge "R72") && ($RELVER ne "RTOOL")){
		push @CCFLAGS, "+DA1.0", "+DS1.0", "-Dunix", "-D_FILE_OFFSET_BITS=64";
		push @CFLAGS, "";
	} else {
		push @CCFLAGS, "+DA1.0", "+DS1.0", "-Dunix";
		push @CFLAGS, "-D_FILE_OFFSET_BITS=64";
	}
	@CCMODEANSI = ("-Ae");
	@CCMODELINT = ("-Ae");
	@CCMODENORM = ("-Ae");
	if ( $RELVER ge "R80" ) {
# force strict ansi scope rules
		push @CXXFLAGS, "-Aa";
	}
        push @CXXFLAGS, "+W930";
} elsif ($MACH eq "PA11") {
	push @CCFLAGS, "-DHP9";
#PTS 1105844
	push @CCFLAGS, "+DA1.1", "+DS2.0", "-Dunix", "-D_FILE_OFFSET_BITS=64";
	push @CFLAGS, "";
	@CCMODEKR = ("-Ac", "-D_HPUX_SOURCE");
    push @CXXFLAGS, "+W930";
    if ($RELVER ge "R80") { push @CXXFLAGS, "-AA"; }
} elsif ($MACH eq "PA20W") {
	push @CCFLAGS, "-DHP9";
	push @CCFLAGS, "+DA2.0W", "+DS2.0", "-Dunix", "-DBIT64";
	@CCMODEKR = ("-Ac", "-D_HPUX_SOURCE");
	push @ASFLAGS, "+DA2.0W", "+DS2.0";
    push @CXXFLAGS, "+W930";
    if ($RELVER ge "R80") { push @CXXFLAGS, "-AA"; }
}
#
#PTS 1107931 Temporary due to compiler bug.....
#PTS 1114586 performance improvement ...
#  push @CXXFLAGS, "+inline_level0";
#
#



if ($ENV{APOMAKE})
{
	@OPTIMIZE = ("+O2", "+Onolimit");
}
# "OPTIMIZE" now depends on "VMAKE_VERSION" (needed on AIX)
elsif ($CURR_VERSION =~ /^f/i) {
    # the old value, it was used for everything
    if ($MACH eq "HP9") {
        @OPTIMIZE = ("+O2");
    } elsif ($MACH eq "PA11" || $MACH eq "PA20W") {
        #PTS 1117629 memset compiler bug ...
        if (($RELVER eq "R74") && ($CORRECTION_LEVEL eq "2"))
        {
           #PTS 1114586 performance improvement ...
           @OPTIMIZE = ("+O2", "+Onolimit");
        }
        else
        {
           #PTS 1114586 performance improvement ...
           @OPTIMIZE = ("+O2", "+Olibcalls" , "+Onolimit");
        }
    }
}
elsif ($CURR_VERSION =~ /^q/i) { @OPTIMIZE = "+O2" }
else {
	# VMAKE_VERSION is quick or slow
	# one level less than fast - not yet really tested
	# the idea is to combine range checks with less risky code
	@OPTIMIZE = ("+O2");
}

# CTS 1103440
@DEBUG_FLAGS = ("-g");


# CTS 1108431
@NO_FLAGS = ();



# cba 98/07/16 : cause of differeces of C- and C++-Linker
# we may have different linker options on some UNIX-platforms
# we use @LDFLAGS for C-linking and @LDXXFLAGS for C++-Linking
# &gar 99/06/03: for linkdll new variable DLDFLAGS and DLDXXFLAGS
# ( PTS 1103001 )
# initialize
# CTS 1107045
# @DLD_CXX_C_FLAGS_START, DLD_CXX_C_FLAGS_END
# @DLD_CXX_C_FLAGS_START  options before objects
# @DLD_CXX_C_FLAGS_END    options after objects
if ($MACH eq "HP9")
{
	@LDFLAGS = @LDXXFLAGS = ("+DA1.0", "+DS1.0");
	push @LDXXFLAGS, "-ldld";
	push @LDFLAGS, "-Wl,-a,archive_shared", "-ldld";
	@KLDFLAGS = ("+DA1.0", "+DS1.0", "-Wl,-a,archive");
	@DLDFLAGS  = @DLDXXFLAGS = ("-Wl,-b");
	# not supported
	@DLD_CXX_C_FLAGS_START=();
	@DLD_CXX_C_FLAGS_END=();
}
elsif ($MACH eq "PA11")
{
	@LDFLAGS = @KLDFLAGS = @LDXXFLAGS = ("+DA1.1", "+DS2.0");
	@DLDXXFLAGS = ("+DA1.1", "+DS2.0", "-b");
	# PTS 1107045
	if ($RELVER ge "R72")
	{
		@DLDFLAGS  = ("-b");
	}
	else
	{
		@DLDFLAGS  = @DLDXXFLAGS;
	}
	@DLD_CXX_C_FLAGS_START =
	("-b", "+nosmartbind", "/opt/aCC/lib/shlrt0.o");
    if ($RELVER lt "R80") {
        @DLD_CXX_C_FLAGS_END =
        ("-L/opt/aCC/lib", "-lCsup", "/opt/aCC/lib/cpprt0.o");
    } else {
        push @LDXXFLAGS, "-AA"; # imply -lCsub_v2
        @DLD_CXX_C_FLAGS_END =
        ("-L/opt/aCC/lib", "/opt/aCC/lib/cpprt0.o");
    }
    if ($RELVER ge "R80") { push @LDXXFLAGS, "-AA"; }
}
elsif ($MACH eq "PA20W")
{
	#PTS 1114586 performance improvement ...
	if ($ENV{APOMAKE})
	{
		@LDFLAGS  = @KLDFLAGS = @LDXXFLAGS = ("+DA2.0W", "-Wl,+pi,256M" , "-Wl,+pd,256M" );
		@DLDFLAGS  = @DLDXXFLAGS = ("+DA2.0W", "-b" ,"-Wl,+pi,256M" , "-Wl,+pd,256M" );
    }
    else
    {
	    @LDFLAGS  = @KLDFLAGS = @LDXXFLAGS = ("+DA2.0W", "+DS2.0" ,
	      "-Wl,+pi,256M" , "-Wl,+pd,256M" );
		@DLDFLAGS  = @DLDXXFLAGS = ("+DA2.0W", "+DS2.0", "-b" ,
	      "-Wl,+pi,256M" , "-Wl,+pd,256M" );
    }
	# PTS 1107045
	if ($RELVER ge "R72")
	{
		@DLDFLAGS  = ("-b");
	}
	else
	{
		@DLDFLAGS  = @DLDXXFLAGS;
	}
	@DLD_CXX_C_FLAGS_START = ("-b", "-a", "archive_shared");
    if ($RELVER lt "R80") {
        @DLD_CXX_C_FLAGS_END = (
        "-L/usr/lib/pa20_64",
        "-L/opt/langtools/lib/pa20_64",
        "-lCsup", "-lcl");
    } else {
        push @LDXXFLAGS, "-AA";  # imply -lCsub_v2
        @DLD_CXX_C_FLAGS_END = (
        "-L/usr/lib/pa20_64", "-L/opt/langtools/lib/pa20_64", "-lcl");
    }
}

#############################
# options for Multipasslink #
#############################

@KEEP_SYMBOLS_FIRST_OPTION = ();
@KEEP_SYMBOLS_OPTION_BEFORE_LIB = ();
@KEEP_SYMBOLS_OPTION_AFTER_LIB = ();

@KEEP_SYMBOLS_FIRST_OPTION=("-Wl,+n" );

#
# For relocatable objects (collection of object files) a different linker is used
# These definitions replace system dependend setting in linkrel.pl!
#
# WIN32 uses AR and AS_FLAGS
#
# SUN:
# There is no specific option that tells ld   to  link  64-bit
# objects;  the  class of the first object that gets processed
# by ld  determines whether it is to perform  a  32-bit  or  a
# 64-bit link edit.
# AIX:
# 32bit and 64bit relocatable object may even coexist...
#
	@LDREL_FLAGS = ( "-r", "-L$TOOL_LIB" );

# &jrg 14.9.1999
#
# C++ compilation sometimes needs to setup a path to a directory for
# the template cache. In this template cache instances of template
# functions with external linkage are stored.
# Since template instances may differ depending on
# VMAKE_VERSION (fast,slow...) for each version a seperate
# template cache is forced
#
#
# nothing needed... +inst_all is default. Linker seems intelligent enough to
# eliminate doubled template definitions or produces LOTs of code...
#

# &gar PTS 1001828
@PRF_FLAGS = ();

# &gar OFLAGS for call options
# used in link... and comp...
# always initialize here with ()
@OFLAGS=();

$SR_ID =   "static char ident_sr[] =";
$MF_ID =   "static char ident_mf[] =";
$PC_ID =   "static char ident_pc[] =";
$CC_ID  =  "static char ident_cc[] =";
$ID_END = ";";
$BUILD_ID      = "#ident";
$BUILD_ID_END  = "";

if (($MACH eq "HP9") || ($MACH eq "PA11")) {
	@PCFLAGS= ( "-qn", "-e" );
} elsif ($MACH eq "PA20W") {
	@PCFLAGS= ( "-qn", "-e" , "-64" );
}
@ARFLAGS = ( "rcs" );
$RANLIB = "ar ts";

if ($ENV{APOMAKE})
{	@LIBPC=(); }
else
{
	@LIBPC = ( "$TOOL_LIB/libsqlptc.a" );	# static, not dynamic lib
	@LIBPC_PROF = ( "$TOOL_LIB/libsqlptc.a");	# static, not dynamic lib
}



if ($MACH eq "HP9") {
	@LIBSCR = "-lHcurses";
} else {
	@LIBSCR = "-lcurses";
}
@LIBMATH = "-lm";
@DYNAMIC_LIB = ();
@LIBNET = ();
@LIBCSCI    = ();

#
# PTS 1002837 jrg 26.5.1999
@LIBMPROC = ();

# CTS 1106257
@LIBLD=();
if (( $RELVER ge "R72") && ($RELVER ne "RTOOL"))
{	@LIB_PTHREAD=("-lpthread"); }
else
{	@LIB_PTHREAD=(); }
@LIBAIO = ();



# &gar PTS 1106517
@LDSHMFLAGS=();

# jrg PTS 1110802
@LDSHMLIBS=();

if ( $SQLVERSION eq "REL30" )  { $TRACETYPE = "tsp_trace" }
	else { $TRACETYPE = "tracetext" }


##########
$PATH_SEPARATOR=":";

###########
# $PREPROCESS_FLAGS
@PP_FLAGS=("-P");

###########
if ($ENV{APOMAKE})
{
	$SYMBOL_DIR = "$INSTROOT/sap";
	$MAP_DIR    = "$INSTROOT/sap";
}
else
{
	$SYMBOL_DIR = "$INSTROOT/symbols";
	$MAP_DIR    = "$INSTROOT/Support/Mapfiles";
}

$SYMBOL_DIR = $ENV{FORCED_SYMBOL_DIR} if ($ENV{FORCED_SYMBOL_DIR});
$MAP_DIR = $ENV{FORCED_MAP_DIR} if ($ENV{FORCED_MAP_DIR});


#################################################
# vmake environment variables must be set by hand
#################################################

if ( ! $DEVENV ) {
#   $DEVENV = "YES";		# Pascal vsp-modules
    $DEVENV = "NO";		# Assembler or C vsp-modules
}
$ENV{DEVENV} = $DEVENV; 	# export $DEVENV

undef($CCOMP);
$CCOMP = "GENC";
$ENV{CCOMP} = $CCOMP;
undef($CPPCOMP);
undef($COBCOMP);
undef($FORCOMP);
undef($PASCOMP);


# ================================================================

$DLL_TYPE = "so";

$DLL_TYPE = "sl";
if ($MACH eq "HP9" || $MACH eq "PA11") {
	push @CCFLAGS, "+Z";		# "position independent code" required
}
$ENV{DLL_TYPE} = $DLL_TYPE;		# can be exported even if not set

$ARCH_TYPE = ".a";
$ENV{ARCH_TYPE} = $ARCH_TYPE;		# can be exported even if not set

#################################################
#  IGNORE following strings in output
#  separate with "|"
#################################################

# string for compile
#
$CC_IGNORE="";

# string for link
$LNK_IGNORE="";

sub setvar {
    my ($varname, $commonDefault, $internalDefault) = @_;
    if (!defined ($internalDefault)) {
        $internalDefault = $commonDefault;
    }
    my ($value);
    if ($ENV{$varname}) {
        $value = $ENV{$varname};
    }
    else {
        if ($ENV{SAPDB_INTERNAL}) {
            $value = $internalDefault;
        }
        else {
            $value = $commonDefault;
        }
        $ENV{$varname} = $value;
    }
    eval '$' . "$varname = '$value'";
}

setvar ('PYTHON', 'NO', "$TOOL/Python");
setvar ('PYTHON_INCLUDE', $ENV{PYTHON} . '/include');
setvar ('ACTIVEPERL_INCLUDE', 'NO', 'NO');
setvar ('ACTIVEPERL_LIB', 'NO', 'NO');
setvar ('COREPERL_INCLUDE', $Config{'archlib'} . "/CORE");
setvar ('COREPERL_LIB', $Config{'archlib'} . "/CORE");
setvar ('JDK_INCLUDE', 'NO', "$TOOL/java/include");
setvar ('JAVA_HOME', 'NO', "$TOOL/java");

# set installation directory for libraries and scripting interfaces
if ($ENV{BIT64} eq "1" && $MACH ne "ALPHA" && $MACH ne "IA64")
{
    #all 64-bit-platforms should use lib/lib64
    $ENV{INSTLIB} = $INSTROOT."/lib/lib64";
    $ENV{INSTMISC} = $INSTROOT."/misc/lib64";
} else {
    $ENV{INSTLIB} = $INSTROOT."/lib";
    $ENV{INSTMISC} = $INSTROOT."/misc";
}

unless (defined $ENV{INSTRUNTIME}) {
	require GetInstDir;
	$ENV{INSTRUNTIME} = GetInstDir::GetInstRuntime ();
}

unless (defined $ENV{SAPDBSDK}) {
	require GetInstDir;
	$ENV{SAPDBSDK} = GetInstDir::GetSDKRoot ();
}

unless (defined $ENV{SAPDBSDKLIB}) {
	require GetInstDir;
	$ENV{SAPDBSDKLIB} = GetInstDir::GetSDKRootLib ();
}

unless (defined $ENV{SAPDBSDKPGM}) {
	require GetInstDir;
	$ENV{SAPDBSDKPGM} = GetInstDir::GetSDKRootPgm ();
}

$TraceFile = "$WRK/$VMAKE_VERSION/prot/trace.prot";
sub ToolTrace {
	# usage ToolTrace($string, \@daten)
	if ($ENV{TOOLTRACE}) {
		local($string, $r_daten) = @_;
		open(TOOLTRACE, ">>$TraceFile") ||
			die "Can't open $TraceFile (append): $!\n";
		print TOOLTRACE sprintf($string, @$r_daten);
		close(TOOLTRACE);
	}
}

#
# Override toolvars variables through env variable
#    example: export TOOLVARS_PATCH="OS=WIN32;OSSPEC=WIN32;MACH=I386"
#    useful to generate vmake dependencies for any platform without
#    remote compilation
#
if ($ENV{TOOLVARS_PATCH}) {
    my @entries = split (';', $ENV{TOOLVARS_PATCH});
    foreach my $entry (@entries) {
        if ($entry =~ /(.*)=(.*)/) {
            eval '$' . $1 . "=" . $2 . ";";
            $ENV{$1} = $2;
        }
    }
}

# initializing flush of output buffer
$|=1; print "";

